import { Edge,Node } from "@xyflow/react";
import { BaseLayout } from "./BaseLayout";
import { LayoutOptions, NodeWithLayout } from "./types";

// 思维导图布局实现
export class MindMapLayout extends BaseLayout {
    layout(options: LayoutOptions): { nodes: Node[], edges: Edge[] } {
        const {
            nodes,
            edges,
            levelSeparation = 200,
            nodeSeparation = 60
        } = options;

        const nodeMap = this.buildNodeMap(nodes);
        const rootNode = this.buildTreeStructure(nodeMap, edges);
        if (!rootNode) return { nodes, edges };

        this.assignSides(rootNode);
        this.calculateSubtreeHeight(rootNode, nodeSeparation);
        this.calculateLayout(rootNode, 0, 0, levelSeparation, nodeSeparation);

        const layoutedNodes = Array.from(nodeMap.values()).map(node => ({
            ...node,
            position: node.position,
        }));

        return { nodes: layoutedNodes, edges };
    }

    private assignSides(node: NodeWithLayout, isRight: boolean = true): void {
        if (!node.children?.length) return;

        const len = node.children.length;
        const midIndex = Math.floor(len / 2);

        if (!node.parent) {
            for (let i = 0; i < len; i++) {
                const child = node.children[i];
                this.assignSides(child, i < midIndex);
                child.isRight = i < midIndex;
            }
        } else {
            node.children.forEach(child => {
                this.assignSides(child, isRight);
                child.isRight = isRight;
            });
        }
    }

    private calculateSubtreeHeight(node: NodeWithLayout, nodeSeparation: number): number {
        if (!node.children?.length) {
            node.subtreeHeight = node.height || 40;
            return node.subtreeHeight;
        }

        const childrenHeight = node.children.reduce((sum, child) => {
            return sum + this.calculateSubtreeHeight(child, nodeSeparation);
        }, 0);

        const totalGaps = (node.children.length - 1) * nodeSeparation;
        node.subtreeHeight = Math.max(node.height || 40, childrenHeight + totalGaps);
        return node.subtreeHeight;
    }

    private calculateLayout(
        node: NodeWithLayout,
        x: number,
        y: number,
        levelSeparation: number,
        nodeSeparation: number
    ): void {
        node.position = { x, y };
        if (!node.children?.length) return;

        let currentY = y - (node.subtreeHeight || 0) / 2;

        node.children.forEach(child => {
            const direction = child.isRight ? 1 : -1;
            const childX = x + (levelSeparation * direction);
            const childY = currentY + (child.subtreeHeight || 0) / 2;

            this.calculateLayout(child, childX, childY, levelSeparation, nodeSeparation);
            currentY += (child.subtreeHeight || 0) + nodeSeparation;
        });
    }
}