import { Edge, Node } from "@xyflow/react";
import { BaseLayout } from "./BaseLayout";
import { LayoutOptions, NodeWithLayout } from "./types";

/**
 * SingleMapLayout 类继承自 BaseLayout，用于实现单图布局。
 * 该类主要负责将节点和边按照一定的规则进行布局，使得节点在视觉上呈现出层次分明、结构清晰的效果。
 */
export class SingleMapLayout extends BaseLayout {
    /**
     * 布局方法，根据提供的选项对节点和边进行布局。
     * @param options 布局选项，包含节点、边、层级间距和节点间距等信息。
     * @returns 返回布局后的节点和边。
     */
    layout(options: LayoutOptions): { nodes: Node[], edges: Edge[] } {
        const { nodes, edges, levelSeparation = 100, nodeSeparation = 30 } = options;
        const nodeMap = this.buildNodeMap(nodes);
        const root = this.buildTreeStructure(nodeMap, edges);

        if (!root) {
            return { nodes: [], edges: [] };
        }

        // 计算子树的尺寸
        this.calculateSubtreeDimensions(root);

        // 第一遍：分配垂直位置
        this.assignInitialVerticalPositions(root, 0);

        // 第二遍：使用平衡布局定位节点
        this.positionNodes(root, 0, 0, levelSeparation, nodeSeparation, 'right');

        return {
            nodes: Array.from(nodeMap.values()),
            edges
        };
    }

    /**
     * 计算子树的尺寸，包括高度和宽度。
     * @param node 当前节点。
     */
    private calculateSubtreeDimensions(node: NodeWithLayout): void {
       
        node.subtreeHeight = node.height || 40;
        node.subtreeWidth = node.width || 150;

        if (node.children && node.children.length > 0) {
            // 首先计算所有子节点的尺寸
            node.children.forEach(child => this.calculateSubtreeDimensions(child));

            // 计算子节点所需的总高度，包括间距
            const totalChildrenHeight = this.calculateTotalChildrenHeight(node.children, 30);

            // 更新节点的子树尺寸
            node.subtreeHeight = Math.max(node.subtreeHeight, totalChildrenHeight);
            node.subtreeWidth += Math.max(...node.children.map(child => child.subtreeWidth || 0));
        }
    }

    /**
     * 计算子节点的总高度。
     * @param children 子节点数组。
     * @param spacing 子节点之间的间距。
     * @returns 返回子节点的总高度。
     */
    private calculateTotalChildrenHeight(children: NodeWithLayout[], spacing: number): number {
        if (!children.length) return 0;
        const totalHeight = children.reduce((sum, child) => sum + (child.subtreeHeight || 0), 0);
        return totalHeight + (spacing * (children.length - 1));
    }

    /**
     * 分配初始垂直位置。
     * @param node 当前节点。
     * @param level 当前层级。
     */
    private assignInitialVerticalPositions(node: NodeWithLayout, level: number): void {
        if (!node.children?.length) return;
        const totalHeight = this.calculateTotalChildrenHeight(node.children, 30);
        let currentY = -(totalHeight / 2);
        node.children.forEach(child => {
            const childHeight = child.subtreeHeight || 0;
            child.verticalLevel = level + 1;
            child.relativeY = currentY + (childHeight / 2);
            this.assignInitialVerticalPositions(child, level + 1);
            currentY += childHeight + 30; // 30 是垂直间距
        });
    }

    /**
     * 定位节点。
     * @param node 当前节点。
     * @param x 当前节点的水平位置。
     * @param y 当前节点的垂直位置。
     * @param levelSeparation 层级间距。
     * @param nodeSeparation 节点间距。
     * @param direction 布局方向，'left' 或 'right'。
     */
    private positionNodes(
        node: NodeWithLayout,
        x: number,
        y: number,
        levelSeparation: number,
        nodeSeparation: number,
        direction: 'left' | 'right'
    ): void {
        node.position = { x, y };
        if (!node.children?.length) return;
        // 计算子节点的水平位置
        const nextX = direction === 'right'
            ? x + (node.width || 0) + levelSeparation
            : x - (node.width || 0) - levelSeparation;
        // 定位每个子节点
        node.children.forEach(child => {
            const childY = y + (child.relativeY || 0);
            this.positionNodes(
                child,
                nextX,
                childY,
                levelSeparation,
                nodeSeparation,
                direction
            );
        });
    }
}