import { BaseLayout } from "./BaseLayout";
import { LayoutOptions, LayoutStrategy, NodeWithLayout } from "./types";
import { Edge, Node } from "@xyflow/react";
export class TreeLayout extends BaseLayout {
    layout(options: LayoutOptions): { nodes: Node[], edges: Edge[] } {
        const {
            nodes,
            edges,
            levelSeparation = 100,  // 层级间垂直距离
            nodeSeparation = 50     // 节点间水平距离
        } = options;

        const nodeMap = this.buildNodeMap(nodes);
        const rootNode = this.buildTreeStructure(nodeMap, edges);
        if (!rootNode) return { nodes, edges };
        // 计算每个节点的子树宽度
        this.calculateSubtreeWidth(rootNode, nodeSeparation);
        // 计算布局位置
        this.calculateTreeLayout(rootNode, 0, 0, levelSeparation);
        const layoutedNodes = Array.from(nodeMap.values()).map(node => ({
            ...node,
            position: node.position,
        }));
        return { nodes: layoutedNodes, edges };
    }

    private calculateSubtreeWidth(node: NodeWithLayout, nodeSeparation: number): number {
        if (!node.children?.length) {
            node.subtreeWidth = node.width || 150;
            return node.subtreeWidth;
        }

        const childrenWidth = node.children.reduce((sum, child) => {
            return sum + this.calculateSubtreeWidth(child, nodeSeparation);
        }, 0);

        const totalGaps = (node.children.length - 1) * nodeSeparation;
        node.subtreeWidth = Math.max(node.width || 150, childrenWidth + totalGaps);
        return node.subtreeWidth;
    }

    private calculateTreeLayout(
        node: NodeWithLayout,
        x: number,
        y: number,
        levelSeparation: number
    ): void {
        node.position = { x, y };

        if (!node.children?.length) return;

        const totalChildrenWidth = node.children.reduce((sum, child) =>
            sum + (child.subtreeWidth || 0), 0);
        const totalGaps = (node.children.length - 1) * (node.width || 150);

        // 计算最左侧子节点的起始x坐标
        let startX = x - (totalChildrenWidth + totalGaps) / 2;

        node.children.forEach(child => {
            const childX = startX + (child.subtreeWidth || 0) / 2;
            const childY = y + levelSeparation;

            this.calculateTreeLayout(child, childX, childY, levelSeparation);

            startX += (child.subtreeWidth || 0) + (node.width || 150);
        });
    }
}