import { memo, useCallback, useEffect, useRef, useState } from 'react';
import { Handle, Position, NodeProps, Node, useUpdateNodeInternals } from '@xyflow/react';
import useGraphStore from '../store';
import { shallow } from 'zustand/shallow';
import { GraphState } from '../types';
import { cn } from '@web/src/utils/classname';
import { LEVEL_STYLES, NODE_BASE_STYLES, TEXTAREA_BASE_STYLES } from './style';

export type GraphNode = Node<{
    label: string;
    color?: string;
    level?: number;
}, 'graph-node'>;



interface TextMeasurerProps {
    element: HTMLTextAreaElement;
    minWidth?: number;
    maxWidth?: number;
    padding?: number;
}

const measureTextWidth = ({
    element,
    minWidth = 60,
    maxWidth = 400,
    padding = 16,
}: TextMeasurerProps): number => {
    const span = document.createElement('span');
    const styles = {
        visibility: 'hidden',
        position: 'absolute',
        whiteSpace: 'pre',
        fontSize: window.getComputedStyle(element).fontSize,
    } as const;

    Object.assign(span.style, styles);
    span.textContent = element.value || element.placeholder;
    document.body.appendChild(span);

    const contentWidth = Math.min(Math.max(span.offsetWidth + padding, minWidth), maxWidth);
    document.body.removeChild(span);

    return contentWidth;
};

const selector = (store: GraphState) => ({
    updateNode: store.updateNode,
});

export const GraphNode = memo(({ id, selected, width, height, data, isConnectable }: NodeProps<GraphNode>) => {
    const { updateNode } = useGraphStore(selector, shallow);
    const [isEditing, setIsEditing] = useState(false);
    const [inputValue, setInputValue] = useState(data.label);
    const [isComposing, setIsComposing] = useState(false);
    const containerRef = useRef<HTMLDivElement>(null);
    const textareaRef = useRef<HTMLTextAreaElement>(null);
    const updateNodeInternals = useUpdateNodeInternals();
    // const [nodeWidth, setNodeWidth] = useState(width)
    // const [nodeHeight, setNodeHeight] = useState(height)
    const updateTextareaSize = useCallback((element: HTMLTextAreaElement) => {
        const contentWidth = measureTextWidth({ element });
        element.style.whiteSpace = contentWidth >= 400 ? 'pre-wrap' : 'pre';
        element.style.width = `${contentWidth}px`;
        element.style.height = 'auto';
        element.style.height = `${element.scrollHeight}px`;

    }, []);

    const handleChange = useCallback((evt: React.ChangeEvent<HTMLTextAreaElement>) => {
        const newValue = evt.target.value;
        setInputValue(newValue);
        updateNode(id, { label: newValue });
        updateTextareaSize(evt.target);
    }, [updateNode, id, updateTextareaSize]);

    useEffect(() => {
        if (textareaRef.current) {
            updateTextareaSize(textareaRef.current);
        }
    }, [isEditing, inputValue, updateTextareaSize]);

    const handleKeyDown = useCallback((evt: React.KeyboardEvent<HTMLTextAreaElement>) => {
        const isAlphanumeric = /^[a-zA-Z0-9]$/.test(evt.key);
        const isSpaceKey = evt.key === ' ';

        if (!isEditing && (isAlphanumeric || isSpaceKey)) {
            evt.preventDefault();
            evt.stopPropagation();

            const newValue = isAlphanumeric ? evt.key : data.label;
            setIsEditing(true);
            setInputValue(newValue);
            updateNode(id, { label: newValue });
            return;
        }

        if (isEditing && evt.key === 'Enter' && !evt.shiftKey && !isComposing) {
            evt.preventDefault();
            setIsEditing(false);
        }
    }, [isEditing, isComposing, data.label, id, updateNode]);

    const handleDoubleClick = useCallback(() => {
        setIsEditing(true);
    }, []);

    const handleBlur = useCallback(() => {
        setIsEditing(false);
    }, []);
    useEffect(() => {
        const resizeObserver = new ResizeObserver((entries) => {
            for (const entry of entries) {
                const { width, height } = entry.contentRect;
                updateNodeInternals(id);
            }
        });

        if (containerRef.current) {
            resizeObserver.observe(containerRef.current);
        }

        return () => {
            resizeObserver.disconnect();
        };
    }, []);
    return (
        <div
            ref={containerRef}
            onDoubleClick={handleDoubleClick}
            className={cn(
                NODE_BASE_STYLES,
                LEVEL_STYLES[data.level ?? 2].container,
                selected && 'ring-2 ring-blue-400',
                isEditing && 'ring-2 ring-blue-500'
            )}
            data-testid="graph-node"
        >
            <textarea
                ref={textareaRef}
                value={inputValue}
                onChange={handleChange}
                onBlur={handleBlur}
                onKeyDown={handleKeyDown}
                onCompositionStart={() => setIsComposing(true)}
                onCompositionEnd={() => setIsComposing(false)}
                className={cn(
                    TEXTAREA_BASE_STYLES,
                    LEVEL_STYLES[data.level ?? 2].fontSize,
                    isEditing ? 'nodrag' : 'cursor-default'
                )}
                placeholder={isEditing ? "输入节点内容..." : "双击编辑"}
                rows={1}
                readOnly={!isEditing}
                aria-label="节点内容"
            />
            <Handle
                type="source"
                position={Position.Left}
                isConnectable={isConnectable}
                id="source"
                style={{ left: 0 }}
                className="w-3 h-3 bg-blue-400 border-2 border-white rounded-full"
            />
            <Handle
                type="target"
                position={Position.Right}
                isConnectable={isConnectable}
                id="target"
                style={{ right: 0 }}
                className="w-3 h-3 bg-blue-400 border-2 border-white rounded-full"
            />

        </div>
    );
});

GraphNode.displayName = 'GraphNode';